#!/bin/bash

PATH_SAVE_DIR=/tmp/savedPaths
NECESSITAS_SDK_PATH_FILE=$PATH_SAVE_DIR/necessitasSdkPath
NDK_ROOT_PATH_FILE=$PATH_SAVE_DIR/ndkRootPath
ANDROID_SDK_PATH_FILE=$PATH_SAVE_DIR/androidSdkPath
ANT_BIN_PATH_FILE=$PATH_SAVE_DIR/antBinPath


if [ -f $NECESSITAS_SDK_PATH_FILE ]
	then
	NECESSITAS_SDK_PATH=$(cat $NECESSITAS_SDK_PATH_FILE)
else 	
	echo "NECESSITAS_SDK_PATH Not Set !!"
	echo "Please run configure_necessitas.sh atleast once on this machine"
	exit -1
fi

if [ -f $NDK_ROOT_PATH_FILE ]
	then
	NDK_ROOT=$(cat $NDK_ROOT_PATH_FILE)
else 	
	echo "NDK_ROOT Not Set !!"
	echo "Please run configure_necessitas.sh atleast once on this machine"
	exit -1
fi

if [ -f $ANDROID_SDK_PATH_FILE ]
	then
	ANDROID_SDK_PATH=$(cat $ANDROID_SDK_PATH_FILE)
else 	
	echo "ANDROID_SDK_PATH Not Set !!"
	echo "Please run configure_necessitas.sh atleast once on this machine"
	exit -1
fi

if [ -f $ANT_BIN_PATH_FILE ]
	then
	ANT_BIN_PATH=$(cat $ANT_BIN_PATH_FILE)
else 	
	echo "ANT_BIN_PATH Not Set !!"
	echo "Please run configure_necessitas.sh atleast once on this machine"
	exit -1
fi


ANDROID_NDK_PLATFORM=android-8
NDK_HOST=linux-x86
NDK_TOOLCHAIN_PREFIX=arm-linux-androideabi
NDK_TOOLCHAIN_VERSION=4.4.3
TARGET_ARCH=armeabi-v7a		# armeabi, armeabi-v7a
DOWNLOAD_OPENSSL=0
USE_EXCEPTIONS=1
QT_CLEAN=1
QT_BUILD_DEBUG=0

QT_BUILD=1				# 0 - don't build qt
						# 1 - Build qt
						
QT_BUILD_OPT=1			# 0 - don't configure qt (only compile) default
						# 1 - configure qt and compile qt
						
QT_BUILD_TYPE=1			# 0 - Build static version of qt
						# 1 - Build shared version of qt
						
QT_INSTALL=0			# 0 - don't install
                   		# 1 - install

RUNNING_DIR_PATH=$PWD
BASIC_QTCORE_NAME="QtCore"
BASIC_QTXML_NAME="QtXml"
BASIC_QTNW_NAME="QtNetwork"

OUTPUT_SO_QTCORE_NAME="lib$BASIC_QTCORE_NAME.so"
OUTPUT_SO_QTXML_NAME="lib$BASIC_QTXML_NAME.so"
OUTPUT_SO_QTNW_NAME="lib$BASIC_QTNW_NAME.so"

# create the build directory
rm -rf $NECESSITAS_SDK_PATH/Android/Qt/480/$TARGET_ARCH
mkdir -p $NECESSITAS_SDK_PATH/Android/Qt/480/$TARGET_ARCH
cd $NECESSITAS_SDK_PATH/Android/Qt/480/$TARGET_ARCH

QT_INSTALL_PATH=$NECESSITAS_SDK_PATH/Android/Qt/480/$TARGET_ARCH

QT_MODIFY_INSTALL_PATH=0	# 0 - use default
                   			# 1 - use $QT_INSTALL_PATH

export PATH=$PATH:$ANDROID_SDK_PATH/tools
export PATH=$PATH:$ANDROID_SDK_PATH/platform-tools
export PATH=$PATH:$ANT_BIN_PATH

export LD_LIBRARY_PATH=$NECESSITAS_SDK_PATH/Android/Qt/480/$ARM_EABI/lib
export QTDIR=$NECESSITAS_SDK_PATH/Android/Qt/480/$ARM_EABI

# build and install qt
$NECESSITAS_SDK_PATH/Android/Qt/480/qt-src/android/androidconfigbuild.sh -b $QT_BUILD  -q $QT_BUILD_OPT -k $QT_INSTALL -m $QT_MODIFY_INSTALL_PATH -i $QT_INSTALL_PATH -c $QT_CLEAN -d $QT_BUILD_DEBUG -n $NDK_ROOT -v $NDK_TOOLCHAIN_VERSION -f $NDK_TOOLCHAIN_PREFIX -o $NDK_HOST -a $TARGET_ARCH -l `echo $ANDROID_NDK_PLATFORM | grep -Po "(?<=android\-).*"` -w $DOWNLOAD_OPENSSL -x $USE_EXCEPTIONS -h $QT_BUILD_TYPE


# copy libs and headers
OUTPUT_DIR=$RUNNING_DIR_PATH/output
mkdir -p $OUTPUT_DIR
mkdir -p $OUTPUT_DIR/libs
mkdir -p $OUTPUT_DIR/headers

SO_QTCORE_NAME=`find $QT_INSTALL_PATH/lib -type f -name lib$BASIC_QTCORE_NAME.so*`
SO_QTXML_NAME=`find $QT_INSTALL_PATH/lib -type f -name lib$BASIC_QTXML_NAME.so*`
SO_QTNW_NAME=`find $QT_INSTALL_PATH/lib -type f -name lib$BASIC_QTNW_NAME.so*`

#-------------------------------------------------------------------------------#

# copy libs to local dir
cp $SO_QTCORE_NAME $OUTPUT_DIR/libs/$OUTPUT_SO_QTCORE_NAME
cp $SO_QTXML_NAME $OUTPUT_DIR/libs/$OUTPUT_SO_QTXML_NAME
cp $SO_QTNW_NAME $OUTPUT_DIR/libs/$OUTPUT_SO_QTNW_NAME

# copy libs to NDK dir 
cp $SO_QTCORE_NAME $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/lib/$OUTPUT_SO_QTCORE_NAME
cp $SO_QTXML_NAME $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/lib/$OUTPUT_SO_QTXML_NAME
cp $SO_QTNW_NAME $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/lib/$OUTPUT_SO_QTNW_NAME

#-------------------------------------------------------------------------------#

# copy headers to local dir
rm -R $OUTPUT_DIR/headers/ 
mkdir -p $OUTPUT_DIR/headers/$BASIC_QTCORE_NAME
mkdir -p $OUTPUT_DIR/headers/$BASIC_QTXML_NAME
mkdir -p $OUTPUT_DIR/headers/$BASIC_QTNW_NAME
cp -R $QT_INSTALL_PATH/include/$BASIC_QTCORE_NAME/*.* $OUTPUT_DIR/headers/$BASIC_QTCORE_NAME/
cp -R $QT_INSTALL_PATH/include/$BASIC_QTXML_NAME/*.* $OUTPUT_DIR/headers/$BASIC_QTXML_NAME/
cp -R $QT_INSTALL_PATH/include/$BASIC_QTNW_NAME/*.* $OUTPUT_DIR/headers/$BASIC_QTNW_NAME/


# copy headers to NDK dir 
rm -R $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTCORE_NAME
rm -R $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTXML_NAME
rm -R $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTNW_NAME
mkdir -p $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTCORE_NAME
mkdir -p $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTXML_NAME
mkdir -p $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTNW_NAME
cp -R $QT_INSTALL_PATH/include/$BASIC_QTCORE_NAME/*.* $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTCORE_NAME/
cp -R $QT_INSTALL_PATH/include/$BASIC_QTXML_NAME/*.* $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTXML_NAME/
cp -R $QT_INSTALL_PATH/include/$BASIC_QTNW_NAME/*.* $NDK_ROOT/platforms/$ANDROID_NDK_PLATFORM/arch-arm/usr/include/$BASIC_QTNW_NAME/

#-------------------------------------------------------------------------------#
echo -e "DONE"
